package LayoutManagerExample;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * class to demonstrate 3 basic Layout Managers
 * these include GridLayout, FlowLayout, BorderLayout
 * 
 * @author mhotan
 *
 */
public class LMExample {

	//Main frame to hold all my components
	JFrame frame;
	
	//Panels Containers to hold components
	// and demonstrate layout managers
	JPanel flowPanel, gridPanel;
	
	//Components
	JButton[] flowButtons, gridButtons;
	JTextField stretchVert, display; 
	
	/**
	 * Constructs a LMExample application.
	 * Runs application until frame is closed
	 * 
	 * @effects all buttons and Panel
	 */
	public LMExample(){
		//Lets use some helper methods to initial 
		InitializeButtonArrays();
		InitializeComponents();
		
		// Now that everything is initialized
		addButtonsToPanel(flowPanel, flowButtons);
		addButtonsToPanel(gridPanel, gridButtons);
		
		//Ok now lets put our Panels together
		//Lets add our panels to the frame
		frame.add(flowPanel, BorderLayout.SOUTH);
		frame.add(gridPanel, BorderLayout.CENTER);
		frame.add(display, BorderLayout.NORTH);
		frame.add(stretchVert, BorderLayout.WEST);
		//frame.add(stretchVert, BorderLayout.EAST); // What happens if I uncomment this line
		
		frame.pack(); // Fit everything nice and tight
		frame.setVisible(true);
	}


	//Add every button to panel
	private void addButtonsToPanel(JPanel panel, JButton[] buttons) {
		for(JButton button : buttons){
			panel.add(button);
		}
	}

	//General Initialize components function
	private void InitializeComponents() {
		//Initialize Stretch Buttons
		stretchVert = new JTextField("Stretch me Vertically");
		stretchVert.setBackground(Color.GREEN);
		
		// Display
		display = new JTextField("Cmon, Push a button I dare you");
		display.setFont(new Font("Comic Sans MS", Font.BOLD, 14)); //More Comic Sans ;-P
		display.setBackground(Color.BLUE);
		
		//Initial frame setup
		frame = new JFrame("Layout Managing Time");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); 
		frame.setSize(500, 500); //hard code screen size
		frame.setLocationRelativeTo(null); //Center frame on screen
		frame.setLayout(new BorderLayout()); //Assign layout manager
		
		//Lets set Up some panels
		flowPanel = new JPanel(new FlowLayout());
		gridPanel = new JPanel(new GridLayout(3,3));
	}


	private void InitializeButtonArrays() {
		
		//Buttons for flow layout
		flowButtons = new JButton[5];
		
		//Buttons for grid layout
		gridButtons = new JButton[9];
		
		//Initialize all buttons in arrays
		ButtonPrintListener listener = new ButtonPrintListener();
		for (int i=0; i<gridButtons.length; i++){
			gridButtons[i] = new JButton("Grid Button " + (i+1));
			gridButtons[i].addActionListener(listener); //Register listener
		}
		
		//Initialize flow buttons
		for (int i=0; i<flowButtons.length; i++){
			flowButtons[i] = new JButton("Flow Button " + (i+1));
			flowButtons[i].addActionListener(listener); //Register listener
		}
		
	}
	
	/**
	 * Button display Listener
	 * This class is takes the text displayed on each button
	 * and displays it application dsiplay window
	 * 
	 * @requires registered with JButton
	 * @author mhotan
	 */
	class ButtonPrintListener implements ActionListener{

		@Override
		public void actionPerformed(ActionEvent e) {
		
			//If it is a Button Display correct 
			if(e.getSource().getClass() == JButton.class){
				// As the programmer it is my responsibility 
				// to ensure that i only register this listener with
				// JButtons
				// to restrictive? whats the other option? 
				JButton temp = (JButton)e.getSource();
				
				//Display is field of outer class but look i can still reference it
				display.setText("You pressed " + temp.getText());
			}
			
		}
		
	}
	
}
